package org.seasar.extension.dataset.impl;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;
import java.util.TimeZone;

import jxl.JXLException;
import jxl.Workbook;
import jxl.biff.FormatRecord;
import jxl.write.Boolean;
import jxl.write.DateFormat;
import jxl.write.DateTime;
import jxl.write.Label;
import jxl.write.WritableCell;
import jxl.write.WritableCellFormat;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;

import org.seasar.extension.dataset.DataRow;
import org.seasar.extension.dataset.DataSet;
import org.seasar.extension.dataset.DataSetConstants;
import org.seasar.extension.dataset.DataTable;
import org.seasar.extension.dataset.DataWriter;
import org.seasar.extension.dataset.JXLRuntimeException;
import org.seasar.framework.exception.IORuntimeException;
import org.seasar.framework.util.Base64Util;
import org.seasar.framework.util.FileOutputStreamUtil;
import org.seasar.framework.util.ResourceUtil;
import org.seasar.framework.util.StringConversionUtil;

/**
 * @author taichi S.
 */
public class JXlsWriter implements DataWriter, DataSetConstants {

	private WritableWorkbook  workbook;
	private WritableCellFormat dateStyle;
	private WritableCellFormat base64Style;
	
	
	public JXlsWriter(String path) {
		File file = new File(ResourceUtil.getResourceAsFile("."), path);
		setOutputStream(FileOutputStreamUtil.create(file));
	}
	
	public JXlsWriter(OutputStream out) {
		setOutputStream(out);
	}
	
	public void setOutputStream(OutputStream out) {
		try {
			workbook = Workbook.createWorkbook(out);
		} catch(IOException ex) {
		    throw new IORuntimeException(ex);
		}
		dateStyle = new WritableCellFormat(new DateFormat(DATE_FORMAT));
		base64Style = new WritableCellFormat(new Base64Format());
	}
	
	private class Base64Format extends FormatRecord {
	    public Base64Format() {
	        super.setFormatString(BASE64_FORMAT);
	    }
	}
	
	/**
	 * @see org.seasar.extension.dataset.DataWriter#write(org.seasar.extension.dataset.DataSet)
	 */
	public void write(DataSet dataSet) {
		try {
			for (int i = 0; i < dataSet.getTableSize(); ++i){
				DataTable table = dataSet.getTable(i);
				WritableSheet sheet = workbook.createSheet(table.getTableName(), i);
				
				for (int j = 0; j < table.getColumnSize(); ++j) {
				    sheet.addCell(new Label(j, 0, table.getColumnName(j)));
				}
	
				for (int j = 0; j < table.getRowSize(); ++j) {
					for (int k = 0; k < table.getColumnSize(); ++k) {
						DataRow dataRow = table.getRow(j);
						Object value = dataRow.getValue(k);
						if (value != null) {
						    sheet.addCell(createWritableCell(k, j + 1, value));
						}
					}
				}
			}
			workbook.write();
			workbook.close();
		} catch(JXLException ex){
		    throw new JXLRuntimeException(ex);
		} catch (IOException ex) {
			throw new IORuntimeException(ex);
		}
	}
	
	private WritableCell createWritableCell(int col, int row, Object value) {
	    WritableCell result = null;
	    if (value instanceof Number) {
	        result = new Label(col,row, value.toString());
	    } else if (value instanceof Date) {
	        Date date = (Date) value;
	        result = new DateTime(col, row, new Date(date.getTime() + TimeZone.getDefault().getRawOffset()));
	        result.setCellFormat(dateStyle);
	    } else if (value instanceof byte[]) {
	        result = new Label(col, row, Base64Util.encode((byte[]) value));
	        result.setCellFormat(base64Style);
	    } else if (value instanceof java.lang.Boolean) {
	        result = new Boolean(col, row, ((java.lang.Boolean)value).booleanValue());
	    } else {
	        result = new Label(col, row, StringConversionUtil.toString(value, null));
	    }
	    return result;
	}
	
}
